# The existing parsers are in-memory.  For large data files, like
# swissprot, that requires too much memory.

# On the other hand, the records aren't all that large (there's just a
# lot of them.)  So this will scan the input for each start record
# indicator, up to and including the end record indicator, then pass
# the results to the SAX handler.

# Depends on a file being only a set of records, and records must
# terminated by a unique string (like "//\n").

READHINT = 100000
import string
class RecordReader:
    def __init__(self, infile, terminator, readhint = READHINT):
        self.infile = infile
        self.terminator = terminator
        self._lines = []
        self.readhint = readhint
    def read_record(self):
        lines = self._lines
        terminator = self.terminator
        i = 0
        j = 0
        while 1:
            for j in range(i, len(lines)):
                if lines[j] == self.terminator:
                    s = string.join(lines[:j+1], "")
                    self._lines = lines[j+1:]
                    return s
            # terminator not found, so read more lines
            i = j
            new_lines = self.infile.readlines(self.readhint)
            if not new_lines:
                if not lines:
                    return ''  # end of file
                else:
                    raise AssertionError("cannot find terminator: %s" % \
                                         repr(self.terminator))
            lines.extend(new_lines)
