#
def usage(outfile):
    outfile.write("""\
Usage: toxml [--format <name>]
Convert a file (on stdin) into XML tags (on stdout) using the Martel
format. definition.  If not specified, this will auto-guess the format
""")

import sys, getopt, string
from xml.sax import saxutils
import Martel

class DumpXML(saxutils.Canonizer):
    def __init__(self, writer=sys.stdout):
        saxutils.Canonizer.__init__(self, writer)
        # For writing even outside of a match
        # (All the Martel formats are inside the outermost element,
        # so this isn't absolutely necessary.)
        self.elem_level = 1
    def write_data(self, data):
        data=string.replace(data,"&","&amp;")
        data=string.replace(data,"<","&lt;")
        data=string.replace(data,"\"","&quot;")
        data=string.replace(data,">","&gt;")                                    
        self.writer.write(data)

def main(argv):
    args, files = getopt.getopt(argv[1:], "", ["format=", "help", "version"])
    assert not files, "files may not be specified"

    format = None
    for name, val in args:
        if name == "--format":
            format = val
        elif name == "--help":
            usage(sys.stdout)
            sys.exit(0)
        elif name == "--version":
            print "toxml 0.1 - convert a given file into XML"
            sys.exit(0)
        else:
            raise AssertionError("unknown option: %s" % repr(name))

    if format is None:
        # Should dig up the format directory using Martel.__file__
        # then try to read in the available formats.  Next version :)
        from Martel.formats import \
             swissprot38, PDB_2_1, MDL_10_1996, blastall_2_0_10
        format = swissprot38.format | PDB_2_1.format | MDL_10_1996.format | \
                 blastall_2_0_10.format
    else:
        module = __import__("Martel.formats." + format).formats
        module = getattr(module, format)
        format = module.format

    parser = format.make_parser()
    parser.setErrorHandler(saxutils.ErrorRaiser())
    parser.setContentHandler(DumpXML())
    s = sys.stdin.read(10000000)
    if len(s) >= 10000000:
        raise AssertionError, "you really don't want me to parse a 10MB file"
    parser.parse(s)
    
if __name__ == "__main__":
    main(sys.argv)

