# Documentation I have handy for mol files is from October 1996.

from Martel import Re, Rep, Group, Str, Opt, RepN


def Count(name, count):
    return Group(name, Re("." * count))

# Allow the count to be truncated by the end of line
def EndCount(name, count):
    return Group(name, Re("." * count + \
                          "|(?=.{0,%d}\n).+" % (count-1)))

def StartsFloat(name, count):
    return Group(name, Re(r"(?= *[0123456789+-]*\.)" + "." * count))

def StartsInt(name, count):
    return Group(name, Re(r"(?= *[0123456789+-]* )" + "." * count))

# aaabbblllfffcccsssxxxrrrpppiiimmmvvvvvv
# aaa = num_atoms
#    bbb = num_bonds
#       lll = num_atom_lists
#          fff = obsolete
#             ccc = chiral_flag
#                sss = num_stext_entries
#                   xxx = num_reaction_components
#                      rrr = num_reactants
#                         ppp = num_products
#                            iii = num_intermediates
#                               mmm = num_properties (should be 999)
#                                  vvvvvv = ctab_version

counts_line = Count("num_atoms", 3) + \
              Count("num_bonds", 3) + \
              Count("num_atom_lists", 3) + \
              Count("obsolete", 3) + \
              Count("chiral_flag", 3) + \
              Count("num_stext_entries", 3) + \
              Count("num_reaction_components", 3) + \
              Count("num_reactants", 3) + \
              Count("num_products", 3) + \
              Count("num_intermediates", 3) + \
              Count("num_properties", 3) + \
              Count("ctab_version", 6) + \
              Str("\n")

# xxxxx.xxxxyyyyy.yyyyzzzzz.zzzz aaaddccssshhhbbbvvvHHHrrriiimmmnneee
# xxxxx.xxxx = x (coordinate)
#           yyyyy.yyyy = y (coordinate)
#                     zzzzz.zzzz = z (coordinate)
#                                aaa = atom_symbol
#                                   dd = mass_difference
#                                     cc = charge
#                                       sss = atom_stereo_parity
#                                          hhh = hydrogen_count
#                                             bbb = stereo_care_box
#                                                vvv = valence
#                                                   HHH = HO_designator
#                                                      rrr = reaction_type
#                                       reaction_number = iii
#                                          mapping_number =  mmm
#                                               inv_ret_flag =  nn
#                                             exact_change_flag = eee

atom_line = Count("x", 10) + \
            Count("y", 10) + \
            Count("z", 10) + \
            Str(" ") + \
            EndCount("atom_symbol", 3) + \
            Opt(Count("mass_difference", 2) + \
                Count("charge", 2) + \
                EndCount("atom_stereo_parity", 3) + \
                Opt(Count("hydrogen_count", 3) + \
                    Count("stereo_care_box", 3) + \
                    Count("valence", 3) + \
                    Count("HO_designator", 3) + \
                    Count("reaction_type", 3) + \
                    Count("reaction_number", 3) + \
                    Count("mapping_number", 3) + \
                    Count("inv_ret_flag", 2) + \
                    Count("exact_change_flag", 3)
                    )
                ) + \
            Str("\n")

#atom_block = Group("atom_block", Rep(atom_line))
atom_block = Group("atom_block", RepN(atom_line, "num_atoms"))


# 111222tttsssxxxrrrccc
# 111 = atom_id1
#    222 = atom_id2
#       ttt = bond_type
#          sss = bond_stereo
#             xxx = ignore
#                rrr = bond_topology
#                   ccc = center_status
bond_line = Count("atom_id1", 3) + \
            Count("atom_id2", 3) + \
            Count("bond_type", 3) + \
            Opt( Count("bond_stereo", 3) + \
                 Opt(Count("ignore", 3) + \
                     Count("bond_topology", 3) + \
                     Count("center_status", 3)
                     )
                 ) + \
            Str("\n")

bond_block = Group("bond_block", RepN(bond_line, "num_bonds"))

# aaa kSSSSn 111 222 333 444 555
atom_list = Count("atom_id", 3) + \
            Str(" ") + \
            Count("is_not_normal", 1) + \
            Str("    ") + \
            Count("num_entries", 1) + \
            Re("( (?P<atomic_number>...){1,5}") + \
            Str("\n")

atom_list_block = RepN(atom_list, "num_atom_lists")

# xxxxx.xxxxyyyyy.yyyy
# TTTT....

#stext = Re("(?!\w)(?P<stext_x>..........)(?P<stext_y>.*)\n") + \
#        Re("(?P<stext_text>.*)\n")
stext = Re("(?P<stext_y>.*)\n") + \
        Re("(?P<stext_text>.*)\n")

stext_block = RepN(stext, "num_stext_entries")

# Properties

atom_alias = None
atom_value = None
atom_group = None
skip = None  # There's no way I can support this parameter!

charge = Re("M  CHG(?P<count>...)( (?P<atom_id>...) (?P<charge>...))*\n")

properties = charge

ctab_block = counts_line + atom_block + bond_block + atom_list_block + \
             stext_block + Rep(properties)

header_block = Group("molecule_name", Re(".*\n")) + \
               Group("creation_info", Re(".*\n")) + \
               Group("comment", Re(".*\n"))

# IIPPPPPPPPMMDDYYHHmmddSSssssssssssEEEEEEEEEEEERRRRRR
# (no one really uses this, do they?  Provided just in case, and
# hasn't been tested.)
creation_info_format = Count("initials", 2) + \
                       Count("program_name", 8) + \
                       Count("month", 2) + \
                       Count("day", 2) + \
                       Count("year", 2) + \
                       Count("hour", 2) + \
                       Count("minute", 2) + \
                       Count("dimensional_codes", 2) + \
                       Count("scaling_factor1", 2) + \
                       EndCount("scaling_factor2", 10) + \
                       Opt(EndCount("energy", 12) + \
                           Opt(Count("registry_number", 6))
                           )




end = Re("M  END\n")


molfile_format = header_block + ctab_block + end
format = molfile_format
